SystemOrganization addCategory: #'Tweak-Platforms'!

CObject subclass: #CPlatform
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CPlatform commentStamp: 'ar 12/11/2004 22:31' prior: 0!
Abstract representation of a 'platform' for Tweak. Responsibilities include everything we need to be specific per platform. Platforms can be hierarchical, e.g., a "general" platform as superclass and more specific platforms as subclasses (see the MacOS hierarchy which is provided as an example) as long as the subclasses provide sufficient means to identify themselves (again, see MacOS platform tree).!

CPlatform subclass: #CMacOSPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CMacOSPlatform commentStamp: 'ar 12/11/2004 22:31' prior: 0!
Generic Apple Macintosh platform.!

CMacOSPlatform subclass: #CMacOS9Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CMacOS9Platform commentStamp: 'ar 12/11/2004 22:31' prior: 0!
Specific pre-OSX Mac platform (rarely used these days).!

!CMacOS9Platform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:39'!
isActivePlatform
	^super isActivePlatform and:[Smalltalk osVersion asNumber < 1000]! !

!CMacOSPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:28'!
isActivePlatform
	^Smalltalk platformName = 'Mac OS'! !

!CMacOSPlatform methodsFor: 'keyboard' stamp: 'ar 9/21/2006 18:53'!
codingKeys
	"Answer the default coding keys to be used on this platform."
	^#(
			((ctrl 17) pasteIfTrue)
			((ctrl 3) pasteIfFalse)

			((cmd 2) doIt)
			((cmd 35) printIt)
			((cmd 34) inspectIt)
			((cmd 46) definitionsOfIt)
			((cmd 45) sendersOfIt)
			((cmd 11) browseIt)

			((cmd shift 34) exploreIt)
			((ctrl 34) exploreIt)
		)! !

!CMacOSPlatform methodsFor: 'keyboard' stamp: 'ar 9/21/2006 18:54'!
cursorKeys
	"Answer the default cursor keys to be used on this platform"
	^#(
			((123) moveCursorLeft)
			((124) moveCursorRight)
			((126) moveLineUp)
			((125) moveLineDown)
			((115) moveLineStart)
			((119) moveLineEnd)
			((116) movePageUp)
			((121) movePageDown)
			((shift 123) selectCursorLeft)
			((shift 124) selectCursorRight)
			((shift 126) selectLineUp)
			((shift 125) selectLineDown)
			((shift 115) selectLineStart)
			((shift 119) selectLineEnd)
			((opt 123) moveWordLeft)
			((opt 124) moveWordRight)
			((ctrl 123) moveWordLeft)
			((ctrl 124) moveWordRight)
			((cmd 123) moveWordLeft)	"should be move line start?"
			((cmd 124) moveWordRight)	"should be move line end?"
			((ctrl 126) scrollLineUp)
			((ctrl 125) scrollLineDown)
			((cmd 115) moveTextStart)
			((cmd 119) moveTextEnd)
			((ctrl 115) moveTextStart)
			((ctrl 119) moveTextEnd)
			((opt 115) moveTextStart)
			((opt 119) moveTextEnd)
			((cmd shift 123) selectWordLeft)
			((cmd shift 124) selectWordRight)
			((opt shift 123) selectWordLeft)
			((opt shift 124) selectWordRight)
			((cmd shift 115) selectTextStart)
			((cmd shift 119) selectTextEnd)
			((opt shift 115) selectTextStart)
			((opt shift 119) selectTextEnd)
			((ctrl shift 115) selectTextStart)
			((ctrl shift 119) selectTextEnd)
	).! !

!CMacOSPlatform methodsFor: 'keyboard' stamp: 'bgf 11/11/2009 17:53'!
editingKeys
	"Answer the default editing keys to be used on this platform"
	^#(
			((76) accept)
			((52) accept)
			((36) return)
			((ctrl 36) accept)
			((cmd 36) accept)
			((53) escape)
			((cmd 0) selectAll)
			((cmd 3) findText)
			((cmd 5) findNext)
			((cmd 8) copySelection)
			((cmd 7) cutSelection)
			((cmd 9) pasteSelection)
			((cmd 1) accept)
			((cmd 37) revert)
			((cmd 6) undo)
			((cmd shift 6) redo)
			((51) backspace)
			((117) forwardDelete)
	).! !

CMacOSPlatform subclass: #CMacOSXPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CMacOSXPlatform commentStamp: 'ar 12/11/2004 22:32' prior: 0!
The Apple Mac OSX platform.!

!CMacOSXPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:39'!
isActivePlatform
	^super isActivePlatform and:[Smalltalk osVersion asNumber >= 1000]! !

!CPlatform class methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:04'!
current
	"Answer the current platform"
	^Current! !

!CPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:48'!
initialize	"CPlatform initialize"
	"Initialize the receiver"
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.
	self startUp: true.! !

!CPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:22'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^false! !

!CPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:49'!
shutDown: quitting
	"The system is going down"
	Current ifNotNil:[Current shutDown: quitting].
! !

!CPlatform class methodsFor: 'class initialization' stamp: 'bf 1/9/2006 12:45'!
startUp: resuming
	"Determine the current platform. Use the most specific (in terms of subclasses) platform available."
	| platformClass |
	platformClass := self allSubclasses detect:[:any| any isActivePlatform and:[any subclasses noneSatisfy:[:sub| sub isActivePlatform]]] ifNone:[nil].
	platformClass ifNotNil:[
		| oldData |
		oldData := Current ifNotNil: [Current userData].
		Current := platformClass new.
		Current startUp: resuming.
		oldData ifNotNil: [
			Current userData 
				ifNotNil: [oldData addAll: Current userData].
			Current userData: oldData.
		].
	].! !

!CPlatform methodsFor: 'keyboard' stamp: 'bf 6/7/2005 16:47'!
codingKeys
	"Answer the default coding keys to be used on this platform."
	^#(
			"Control codes: A=1, B=2, etc."
			((ctrl 20 "T") pasteIfTrue)
			((ctrl 6 "F") pasteIfFalse)

			((cmd d) doIt)
			((cmd p) printIt)
			((cmd i) inspectIt)
			((cmd m) definitionsOfIt)
			((cmd n) sendersOfIt)
			((cmd b) browseIt)

			((cmd shift i)  exploreIt)
			((ctrl 9 "I") exploreIt)
	)! !

!CPlatform methodsFor: 'keyboard' stamp: 'bf 7/29/2005 12:22'!
cursorKeys
	"Answer the default cursor keys to be used on this platform"
	^#(
			((28) moveCursorLeft)
			((29) moveCursorRight)
			((30) moveLineUp)
			((31) moveLineDown)
			((1) moveLineStart)
			((4) moveLineEnd)
			((11) movePageUp)
			((12) movePageDown)
			((shift 28) selectCursorLeft)
			((shift 29) selectCursorRight)
			((shift 30) selectLineUp)
			((shift 31) selectLineDown)
			((shift 1) selectLineStart)
			((shift 4) selectLineEnd)
			((cmd 28) moveWordLeft)
			((cmd 29) moveWordRight)
			((cmd 30) scrollLineUp)
			((cmd 31) scrollLineDown)
			((cmd 1) moveTextStart)
			((cmd 4) moveTextEnd)
			((cmd shift 28) selectWordLeft)
			((cmd shift 29) selectWordRight)
			"((cmd shift 30) ---)
			((cmd shift 31) ---)"
			((cmd shift 1) selectTextStart)
			((cmd shift 4) selectTextEnd)
	).! !

!CPlatform methodsFor: 'keyboard' stamp: 'mtf 5/15/2010 11:46'!
editingKeys
	"Answer the default editing keys to be used on this platform"
	^#(
			((3) accept)
			((13) return)
			((27) escape)
			((cmd a) selectAll)
			((cmd f) findText)
			((cmd g) findNext)
			((cmd c) copySelection)
			((cmd x) cutSelection)
			((cmd v) pasteSelection)
			((cmd s) accept)
			((cmd l) revert)
			((cmd z) undo)
			((cmd y) redo)
			((8) backspace)
			((127) forwardDelete)
	).! !

!CPlatform methodsFor: 'initialize' stamp: 'ar 12/11/2004 22:47'!
shutDown: quitting
	"Squeak is shutting down. If this platform requires specific shutdown code, this is a great place to put it."
! !

!CPlatform methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:18'!
standardCodingKeys
	"Answer the standardCodingKeys of the receiver"
	<bewareOf: #standardCodingKeysChanged>
	^self propertyValueAt: #standardCodingKeys! !

!CPlatform methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:18'!
standardCodingKeys: aValue
	"Modify the receiver's standardCodingKeys"
	^self propertyValueAt: #standardCodingKeys put: aValue with: #standardCodingKeysChanged! !

!CPlatform methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:18'!
standardEditingKeys
	"Answer the standardEditingKeys of the receiver"
	<bewareOf: #standardEditingKeysChanged>
	^self propertyValueAt: #standardEditingKeys! !

!CPlatform methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:18'!
standardEditingKeys: aValue
	"Modify the receiver's standardEditingKeys"
	^self propertyValueAt: #standardEditingKeys put: aValue with: #standardEditingKeysChanged! !

!CPlatform methodsFor: 'initialize' stamp: 'ar 12/11/2004 23:20'!
startUp: resuming
	"Squeak is starting up. If this platform requires specific intialization, this is a great place to put it."
	standardEditingKeys := CKeyboardDispatchTable new.
	standardEditingKeys addAll: self cursorKeys.
	standardEditingKeys addAll: self editingKeys.
	standardCodingKeys := CKeyboardDispatchTable new.
	standardCodingKeys addAll: self codingKeys.! !

CPlatform subclass: #CRiscOSPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CRiscOSPlatform commentStamp: 'ar 12/11/2004 22:32' prior: 0!
The Acorn Risc OS platform.!

!CRiscOSPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:27'!
isActivePlatform
	^Smalltalk platformName = 'RiscOS'! !

CPlatform subclass: #CUnixPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CUnixPlatform commentStamp: 'ar 12/11/2004 22:32' prior: 0!
Generic Unix platform.!

!CUnixPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:24'!
isActivePlatform
	^Smalltalk platformName = 'unix'! !

CPlatform subclass: #CWin32Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Platforms'!

!CWin32Platform commentStamp: 'ar 12/11/2004 22:32' prior: 0!
Generic Win32 platform.!

!CWin32Platform class methodsFor: 'class initialization' stamp: 'ar 12/8/2009 21:34'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^SmalltalkImage current platformName = 'Win32'! !

!CWin32Platform methodsFor: 'keyboard' stamp: 'bf 6/7/2005 16:44'!
codingKeys
	"The default coding keys"
	^#(
			((cmd T) pasteIfTrue)
			((cmd F) pasteIfFalse)

			((ctrl D) doIt)
			((ctrl P) printIt)
			((ctrl I) inspectIt)
			((ctrl M) definitionsOfIt)
			((ctrl N) sendersOfIt)
			((ctrl B) browseIt)

			((cmd I)  exploreIt)
	)! !

!CWin32Platform methodsFor: 'keyboard' stamp: 'bf 7/29/2005 12:23'!
cursorKeys
	"The standard cursor keys"
	^#(
			((28) moveCursorLeft)
			((29) moveCursorRight)
			((30) moveLineUp)
			((31) moveLineDown)
			((1) moveLineStart)
			((4) moveLineEnd)
			((11) movePageUp)
			((12) movePageDown)
			((shift 28) selectCursorLeft)
			((shift 29) selectCursorRight)
			((shift 30) selectLineUp)
			((shift 31) selectLineDown)
			((shift 1) selectLineStart)
			((shift 4) selectLineEnd)
			((ctrl 28) moveWordLeft)
			((ctrl 29) moveWordRight)
			((ctrl 30) scrollLineUp)
			((ctrl 31) scrollLineDown)
			((ctrl 1) moveTextStart)
			((ctrl 4) moveTextEnd)
			((ctrl shift 28) selectWordLeft)
			((ctrl shift 29) selectWordRight)
			"((ctrl shift 30) ---)
			((ctrl shift 31) ---)"
			((ctrl shift 1) selectTextStart)
			((ctrl shift 4) selectTextEnd)
	).! !

!CWin32Platform methodsFor: 'keyboard' stamp: 'bgf 11/11/2009 17:39'!
editingKeys
	"The default editing keys"
	^#(
			((3) accept)
			((13) return)
			((27) escape)
			((93) showContextMenu)
			((ctrl A) selectAll)
			((ctrl F) findText)
			((ctrl G) findNext)
			"((ctrl H) replaceText)"
			((ctrl C) copySelection)
			((ctrl X) cutSelection)
			((ctrl V) pasteSelection)
			((ctrl S) accept)
			((ctrl L) revert)
			((ctrl Z) undo)
			((ctrl Y) redo)
			((8) backspace)
			((127) forwardDelete)
	).! !
CPlatform initialize!
